from simpletransformers.question_answering import QuestionAnsweringModel
import json
import os


# Create dummy data to use for training.
train_data = [
    {
        'context': "This is the first context",
        'qas': [
            {
                'id': "00001",
                'is_impossible': False,
                'question': "Which context is this?",
                'answers': [
                    {
                        'text': "the first",
                        'answer_start': 8
                    }
                ]
            }
        ]
    },
    {
        'context':
            "Other legislation followed, including the Migratory Bird Conservation Act of 1929, \
            a 1937 treaty prohibiting the hunting of right and gray whales, \
            and the Bald Eagle Protection Act of 1940. These later laws had a low cost to society—the \
            species were relatively rare—and little opposition was raised",
        'qas': [
            {
                'id': "00002",
                'is_impossible': False,
                'question': "What was the cost to society?",
                'answers': [
                    {
                        'text': "low cost",
                        'answer_start': 225
                    }
                ]
            },
            {
                'id': "00003",
                'is_impossible': False,
                'question': "What was the name of the 1937 treaty?",
                'answers': [
                    {
                        'text': "Bald Eagle Protection Act",
                        'answer_start': 167
                    }
                ]
            }
        ]
    }
]

# Save as a JSON file
os.makedirs('data', exist_ok=True)
with open('data/train.json', 'w') as f:
    json.dump(train_data, f)


# Create the QuestionAnsweringModel
model = QuestionAnsweringModel('distilbert', 'distilbert-base-uncased-distilled-squad',
                               args={'reprocess_input_data': True, 'overwrite_output_dir': True})

# Train the model with JSON file
model.train_model('data/train.json')

# The list can also be used directly
# model.train_model(train_data)

# Evaluate the model. (Being lazy and evaluating on the train data itself)
result, text = model.eval_model('data/train.json')

print(result)
print(text)

print('-------------------')

# Making predictions using the model.
to_predict = [{'context': 'This is the context used for demonstrating predictions.',
               'qas': [{'question': 'What is this context?', 'id': '0'}]}]

print(model.predict(to_predict))




""" Search and scrape wikipedia articles from a chosen category """
import os
# import json
from elasticsearch import Elasticsearch
import wikipediaapi
from slugify import slugify
from .. import constants

try:
    client = Elasticsearch()
except ConnectionRefusedError:
    print("Failed to launch Elstcisearch")


log = constants.logging.getLogger(__name__)


def print_categorymembers(categorymembers, level=0, max_level=1):
    for c in categorymembers.values():
        print("%s: %s (ns: %d)" % ("*" * (level + 1), c.title, c.ns))
        if c.ns == wikipediaapi.Namespace.CATEGORY and level < max_level:
            print_categorymembers(c.categorymembers, level=level + 1, max_level=max_level)


# Save articles in separate .txt files

def save_articles(path=os.path.join(constants.DATA_DIR, "wikipedia"), category='Natural_language_processing'):
    os.makedirs(path, exist_ok=True)
    wiki_wiki = wikipediaapi.Wikipedia('en')
    cat = wiki_wiki.page(f"Category:{category}")

    for key, value in cat.categorymembers.items():
        page = wiki_wiki.page(key)
        slug = slugify(key)
        try:
            with open(f'{path}/{slug}.txt', 'w') as new_file:
                new_file.write(page.title)
                new_file.write('\n')
                new_file.write(page.fullurl)
                new_file.write('\n')
                new_file.write(page.text)
        except Exception as error:
            log.error(f"Error writing document {page.title}: {error}")


def index_dir(path=os.path.join(constants.DATA_DIR, "wikipedia")):

    paths = []
    # r=root, d=directories, f = files
    for r, d, f in os.walk(path):
        for file in f:
            if '.txt' in file:
                paths.append(os.path.join(r, file))

    return paths


# def read_up_on(categories):


class Document:

    def __init__(self, title, text, source):

        self.title = title
        self.text = text
        self.source = source
        self.body = {}
        try:
            self.body = {
                "title": self.title,
                "text": self.text,
                "source": self.source
            }
            log.info(f"Elasticsearch Document JSON created: {self.body}")
        except Exception as error:
            log.warning(f"Document JSON instance error: ", error)

    def insert(self):
        slug = slugify(self.body['title'])
        res = None
        try:
            res = client.index(index=slug, body=self.body)
        except Exception as e:
            log.error(f"Could not create a JSON entry for an article {slug}. error: {e}")
        return res


def search(text, index=''):
    """ Full text search within an ElasticSearch index (''=all indexes) for the indicated text """
    return client.search(index=index,
                         body={"query": {"match": {"text": text}}}
                         )


def test_search():
    res = search(text='text similarity')

    print()
    print('Relevant Articles')
    print('=================')
    for doc in res['hits']['hits']:
        print(doc['_source']['title'])
    print('=================')
    print()
    return res


if __name__ == '__main__':
    print(test_search())



# from django.contrib import admin
# from django.urls import path
from django.conf.urls import url  # , include

from qary_app import views


app_name = 'qary_app'


urlpatterns = [
    # path('', home_view),
    url(r'^$', views.PostListView.as_view(), name='list'),
    url(r'^(?P<pk>[-\w]+)/$', views.PostDetailView.as_view(), name='detail'),
    url(r'answer/', views.reply, name='answer'),
    url(r'question/', views.home_view, name='question'),

]



""" juan_bots describes the data behind a pandas df.plot command with a new df.plot_describe() method """
import regex

import seaborn as sns  # noqa
import matplotlib.pyplot as plt
import pandas as pd

import logging
log = logging.getLogger(__name__)

df = pd.read_csv('http://totalgood.org/midata/teaching/rikeem-u/heights_weights_genders.csv')
df.describe()
# Add Color column with red for Female and blue for Male
df['Color'] = (df['Gender'] == 'Male').apply(lambda x: 'blue' if x else 'red')
# c is color, alpha is opacity (1 - transparency)
fig = df.plot(kind='scatter', x='Height', y='Weight', c='Color', alpha=.3)

# this is how you can display your plot if you want to show someone
plt.show()

# I looked around for the DataFrame in the Figure object but couldn't find it, only the bounding box (min and max values)
print(fig.dataLim)


# Your assignment is to reload the dataframe usind pd.read_html() instead of pd.read_csv()
# The url is 'https://totalgood.org/midata/teaching/rikeem-u/heights_weights_genders.html'
# This is the same as before but with the ".html" file extension.
# and read_html() should return a list of dataframes rather than a single dataframe,
# but this url only has one table, so the list should contain only one dataframe
# Once you download the DataFrame rather than plotting it I'd like you to try to describe using statistics.
# The input to your function is a dataframe and the kwargs that were used in the plot command.
# Here's an example.

def plot_describe(df, **kwargs):
    x_name = kwargs.get('x')
    y_name = kwargs.get('y')  # noqa
    return f"Scatter plot with the x axis ranging from {df[x_name].min()} to {df[x_name].max()}"


class Bot:
    def reply(self, statement):
        """ Generate an invoice or timecard for a project """
        log.warning('Timcard reply in progress...')
        responses = []
        match = regex.match(r'(\bplot[-_.:,;\s]{0,3}describe\b\s*)(.*)', statement.lower())
        if match:
            responses.append((1.0, f"Here's your plot_describe {plot_describe(match.groups()[-1])}"))
        return responses



from django.shortcuts import render
from django.views.generic import ListView, DetailView
from qary_app import models
from qary.clibot import CLIBot
# import re  # noqa


bot = CLIBot(bots=('glossary',))

get_request = None


def home_view(request):

    # function return Query.dict from GET
    global get_request
    get_request = request.GET

    return render(request, "question.html", )


def reply(request):

    # k = str(n[20:-4])
    bot_reply = bot.reply(get_request['n'])

    dict_1 = {'insert': bot_reply, 'Question': get_request['n']}

    return render(request, "answer.html", context=dict_1)


class PostListView(ListView):
    context_object_name = 'posts'
    model = models.Post


class PostDetailView(DetailView):
    context_object_name = 'post_detail'
    model = models.Post
    template_name = 'qary_app/first_app_detail.html'



from django.shortcuts import render
from . import models
from django.urls import reverse
from django.views.generic import (View,
                                  TemplateView, ListView,
                                  DetailView, UpdateView,
                                  CreateView, DeleteView)


# Create your views here.


class IndexView(TemplateView):
  template_name = 'index.html'


class PurposeListView(ListView):
  context_object_name = 'purposes'
  model = models.Purpose


class PurposeDetailView(DetailView):
  context_object_name = 'purpose_detail'
  model = models.Purpose
  template_name = 'swot/purpose_detail.html'


class PurposeCreateView(CreateView):
  fields = ('purpose',)
  model = models.Purpose


class PurposeUpdateView(CreateView):
  fields = ('purpose',)
  model = models.Purpose



"""qarysite URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path
from django.conf.urls import url, include
from qary_app import views


urlpatterns = [
    path('admin/', admin.site.urls),
    url(r'^$', views.home_view),
    url(r'^qary_app/', include('qary_app.urls', namespace='qary_app')),
    url(r'question/', views.home_view, name='question'),
    url(r'answer/', views.reply, name='answer'),

]



""" Universal Sentence Encoding loaded as a lambda function into a Keras model

https://www.dlology.com/blog/keras-meets-universal-sentence-encoder-transfer-learning-for-text-data/

## QUESTION_CATEGORIES

    ABBR - 'abbreviation': expression abbreviated, etc.
    DESC - 'description and abstract concepts': manner of an action, description of sth. etc.
    ENTY - 'entities': animals, colors, events, food, etc.
    HUM - 'human beings': a group or organization of persons, an individual, etc.
    LOC - 'locations': cities, countries, etc.
    NUM - 'numeric values': postcodes, dates, speed,temperature, etc
"""
import re
import pandas as pd
import numpy as np

import tensorflow as tf
import tensorflow_hub as hub

from keras import layers
from keras import Model
from keras import backend as K

# from qa_datasets import load_trec_trainset


# Import the Universal Sentence Encoder's TF Hub module
use_encode = hub.Module("https://tfhub.dev/google/universal-sentence-encoder-large/3")


def encode_texts(texts=["That band rocks!", "That song is really cool."], use_encode=use_encode):
    texts = [texts] if isinstance(texts, str) else texts

    with tf.Session() as session:
        session.run([tf.global_variables_initializer(), tf.tables_initializer()])
        usevectors = session.run(use_encode(texts))

    return usevectors


def get_dataframe(filename='train_5500.txt'):
    lines = open(filename, 'r').read().splitlines()
    data = []
    for i in range(0, len(lines)):
        label = lines[i].split(' ')[0]
        label = label.split(":")[0]
        text = ' '.join(lines[i].split(' ')[1:])
        text = re.sub(r'[^A-Za-z0-9 ,\?\'\"-._\+\!/\`@=;:]+', ' ', text)
        data.append([label, text])

    df = pd.DataFrame(data, columns=['label', 'text'])
    df = df[~df.label.isnull()]
    df = df[df.label.str.len().astype(bool)]
    df['label'] = df.label.astype('category')
    return df


test_reviews = ["It was a decent movie, lots of ups and downs. Good thriller -- horrific, disturbing.",
                "I didn't care for the unheroic Protagonist that chose to win the fight to avoid the melodramatic ending of curing his psychosis."]
test_labels = [0, 1]


def use_lambda(x):
    return use_encode(tf.squeeze(tf.cast(x, tf.string)), signature="default", as_dict=True)["default"]


QA_DF = None  # get_dataframe('train_5500.txt')
QA_CATEGORIES = None  # QA_DF.label.cat.categories.tolist()
TRAIN_TEXTS = None
TRAIN_LABELS = None


def normalize_trainset(df_train=QA_DF):
    global QA_DF
    if df_train is None:
        if QA_DF is None:
            QA_DF = get_dataframe('train_5500.txt')
        df_train = QA_DF
        # get_dataframe('train_5500.txt')
    # df_train = load_trec_trainset()
    # print(df_train.head())
    print(df_train.head())

    train_text = df_train['text'].tolist()
    train_text = np.array(train_text, dtype=object)[:, np.newaxis]
    train_label = np.asarray(pd.get_dummies(df_train.label), dtype=np.int8)
    return train_text, train_label


def build_use_classifier(num_classes=7):
    usevector_shape = (512,)
    input_text = layers.Input(shape=(1,), dtype=tf.string)
    usevector = layers.Lambda(use_lambda, output_shape=usevector_shape)(input_text)
    dense = layers.Dense(256, activation='relu')(usevector)
    pred = layers.Dense(num_classes, activation='softmax')(dense)
    model = Model(inputs=[input_text], outputs=pred)
    model.compile(loss='categorical_crossentropy', optimizer='adam', metrics=['accuracy'])
    return model


def train_model(model, train_texts=TRAIN_TEXTS, train_labels=TRAIN_LABELS, filename='model.h5'):
    global TRAIN_TEXTS, TRAIN_LABELS
    if train_texts is None:
        if TRAIN_TEXTS is None:
            TRAIN_TEXTS, TRAIN_LABELS = normalize_trainset()
        train_texts, train_labels = TRAIN_TEXTS, TRAIN_LABELS
    with tf.Session() as session:
        K.set_session(session)
        session.run(tf.global_variables_initializer())
        session.run(tf.tables_initializer())
        history = model.fit(train_texts, train_labels,
                            # validation_data=(test_text, test_label),
                            epochs=10,
                            batch_size=32)
        model.save_weights(filename)
    print(f'Saved model to {filename}.')
    return history


TEST_TEXTS = ["In what year did the titanic sink ?",
              "What is the highest peak in California ?",
              "Who invented the light bulb ?"]


def test_model(model='model.h5', texts=TEST_TEXTS, categories=QA_CATEGORIES):
    texts = np.array(texts, dtype=object)[:, np.newaxis]
    with tf.Session() as session:
        K.set_session(session)
        session.run(tf.global_variables_initializer())
        session.run(tf.tables_initializer())
        if isinstance(model, str):
            filename = model
            model = build_use_classifier(num_classes=len(categories))
            print(f'Loading model from {filename}')
            model.load_weights(filename)
        predictions = model.predict(texts, batch_size=32)

    predict_logits = predictions.argmax(axis=1)
    predicted_labels = [categories[logit] for logit in predict_logits]
    print(predicted_labels)
    df = pd.DataFrame(predictions)
    df.columns = categories
    df['text'] = [t[0] for t in texts]
    df['label'] = predicted_labels
    return df


def generate_from_use(usevector=[]):
    return "Dummy placeholder text until USE generator is working."


def reply(text):
    match = re.match(r'^\s*([Uu][Ss][Ee]\b)?(.*)$')
    conf = 1e-6
    if match:
        text = match.groups()[1]
        conf = 1.
    usevector = encode_texts([text] if isinstance(text, str) else list(text))
    return [(conf, str(pd.Series(usevector).round(2)))]



from django.conf.urls import url
from swot import views
from django.urls import path

app_name = 'swot'

urlpatterns = [

    url(r'^$', views.PurposeListView.as_view(), name='list'),

    url(r'^(?P<pk>\d+)/$',
        views.PurposeDetailView.as_view(), name='detail'),

    url(r'^create/$', views.PurposeCreateView.as_view(), name='create'),

    url(r'^update/(?P<pk>\d+)/$',
        views.PurposeUpdateView.as_view(), name='update'),

]



""" Transformer based chatbot dialog engine for answering questions """
import logging

from tqdm import tqdm

log = logging.getLogger(__name__)


class DownloadProgressBar(tqdm):
    """ Utility class that adds tqdm progress bar to urllib.request.urlretrieve """

    def update_to(self, b=1, bsize=1, tsize=None):
        if tsize is not None:
            self.total = tsize
        self.update(b * bsize - self.n)



# Generated by Django 3.0.2 on 2020-04-10 03:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Purpose',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('purpose', models.CharField(max_length=256)),
            ],
        ),
        migrations.CreateModel(
            name='SWOT',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('strength', models.CharField(blank=True, default=None, max_length=256)),
                ('strength_rate', models.PositiveIntegerField(default=0, null=True)),
                ('weaknesses', models.CharField(blank=True, default=None, max_length=256)),
                ('weaknesses_rate', models.PositiveIntegerField(default=0, null=True)),
                ('oportinuties', models.CharField(blank=True, default=None, max_length=256)),
                ('oportinuties_rate', models.PositiveIntegerField(default=0, null=True)),
                ('threats', models.CharField(blank=True, default=None, max_length=256)),
                ('threats_rate', models.PositiveIntegerField(default=0, null=True)),
                ('strategy', models.TextField(blank=True, default=None)),
                ('purpose', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='s', to='swot.Purpose')),
            ],
        ),
    ]



#!/usr/bin/env python
# coding: utf-8

# # Project 1 - Text Questions Classification:
# ## Classify a Given Question Text to One of the Six Categories
#
# Description: https://docs.google.com/document/d/14YP8_z1iH4X_eg9wY1HN57HGxlpT53YFMdbe8GeZNUk/edit?usp=sharing
import os
import re
from collections import Counter

import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn import metrics
from sklearn.tree import DecisionTreeClassifier

from keras import layers
from keras.models import Model
from keras import backend as K
import tensorflow as tf
import tensorflow_hub as hub

from ..constants import DATA_DIR
from .. import spacy_language_model

nlp = spacy_language_model.load("en_core_web_lg")

file_name = os.path.join(DATA_DIR, 'trec', 'train_5500.label')

with open(file_name, 'rb') as f:
    txt = f.read()

lines = txt.decode('latin').splitlines()


df = []
for idx, line in enumerate(lines):
    match = re.match('([A-Z]+):([a-z]+)[ ]+(.+)', line)
    # print(match.groups())
    df.append(match.groups())

df = pd.DataFrame(df, columns=['label', 'sublabel', 'question'])

print(df.head())


# ### 2.1 NLP: Word2Vec
# #### Output: A DataFrame of document vectors


vectors = []

for idx, row in df.iterrows():
    doc = nlp(row['question'])
    vectors.append(pd.DataFrame([t.vector for t in doc]).mean())
    # print(pd.DataFrame([t.vector for t in doc]).mean())

df_vec = pd.DataFrame(vectors)
print(df_vec.head())


# ## 3. ML Modelling
# ### 3.1 Logistic Regression: Main Lable Classification

# #### 3.1.1 Training

# In[84]:

vec_train, vec_test, label_train, label_test = train_test_split(
    df_vec, df['label'], test_size=0.25, random_state=0, stratify=df['label'])


# In[85]:


# import the class

# instantiate the model (using the default parameters)
logreg = LogisticRegression(solver='liblinear', multi_class='auto', class_weight='balanced')

# fit the model with data
logreg.fit(vec_train, label_train)


# #### 3.1.2 Inference

# In[86]:


# Predictions
label_pred_test = logreg.predict(vec_test)
label_pred_train = logreg.predict(vec_train)


# #### 3.1.3 Modelling Performance
# #### Confusion Matrix, Accuracy, Precision, Recall, and F1 Score

# In[87]:


classes = list(set(label_test))

print(classes)


# In[88]:


# import the metrics class

cnf_matrix = metrics.confusion_matrix(label_test, label_pred_test, labels=classes)
cnf_matrix = pd.DataFrame(cnf_matrix, columns=classes)
cnf_matrix.rename(index={i: k for i, k in enumerate(classes)}, inplace=True)

print("Note: Columns (predictions) vs. Rows (actuals)\n")
print(cnf_matrix)


# In[89]:


accuracy = metrics.accuracy_score(label_test, label_pred_test)
precision_score = metrics.precision_score(label_test, label_pred_test, average='macro', labels=classes)
recall_score = metrics.recall_score(label_test, label_pred_test, average='macro', labels=classes)
f1_score = metrics.f1_score(label_test, label_pred_test, average='macro', labels=classes)

print("Accuracy:", accuracy)
print("Precision:", precision_score)
print("Recall:", recall_score)
print("Unweighted Average F1 Score:", f1_score)


# In[90]:


accuracy = metrics.accuracy_score(label_train, label_pred_train)
precision_score = metrics.precision_score(label_train, label_pred_train, average='macro', labels=classes)
recall_score = metrics.recall_score(label_train, label_pred_train, average='macro', labels=classes)
f1_score = metrics.f1_score(label_train, label_pred_train, average='macro', labels=classes)

print("Accuracy:", accuracy)
print("Precision:", precision_score)
print("Recall:", recall_score)
print("Unweighted Average F1 Score:", f1_score)


# **To compute the metrics on the training data as well

# ### 3.2 Logistic Regression: Sublabel Classification

# In[91]:


vec_train, vec_test, sublabel_train, sublabel_test = train_test_split(
    df_vec, df['sublabel'], test_size=0.25, random_state=0, stratify=df['sublabel'])

# instantiate the model (using the default parameters)
logreg = LogisticRegression(solver='liblinear', multi_class='auto', class_weight='balanced')

# fit the model with data
logreg.fit(vec_train, sublabel_train)

# Predictions
sublabel_pred_test = logreg.predict(vec_test)
sublabel_pred_train = logreg.predict(vec_train)


# In[92]:


classes = list(set(sublabel_test))

print(classes)


# In[93]:


cnf_matrix = metrics.confusion_matrix(sublabel_test, sublabel_pred_test, labels=classes)
cnf_matrix = pd.DataFrame(cnf_matrix, columns=classes)
cnf_matrix.rename(index={i: k for i, k in enumerate(classes)}, inplace=True)

print("Note: Columns (predictions) vs. Rows (actuals)\n")
print(cnf_matrix)


# In[94]:


accuracy = metrics.accuracy_score(sublabel_test, sublabel_pred_test)
precision_score = metrics.precision_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)
recall_score = metrics.recall_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)
f1_score = metrics.f1_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)

print("Accuracy:", accuracy)
print("Precision:", precision_score)
print("Recall:", recall_score)
print("Unweighted Average F1 Score:", f1_score)


# In[95]:


accuracy = metrics.accuracy_score(sublabel_train, sublabel_pred_train)
precision_score = metrics.precision_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)
recall_score = metrics.recall_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)
f1_score = metrics.f1_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)

print("Accuracy:", accuracy)
print("Precision:", precision_score)
print("Recall:", recall_score)
print("Unweighted Average F1 Score:", f1_score)


# ### 3.3 Logistic Regression: Conditional Classification

# In[96]:


for lb in ['ABBR', 'HUM', 'DESC', 'NUM', 'LOC', 'ENTY']:

    vec_train, vec_test, sublabel_train, sublabel_test = train_test_split(
        df_vec[df['label'] == lb], df[df['label'] == lb]['sublabel'],
        test_size=0.25, random_state=0, stratify=df[df['label'] == lb]['sublabel'])

    # instantiate the model (using the default parameters)
    logreg = LogisticRegression(solver='liblinear', multi_class='auto', class_weight='balanced')

    # fit the model with data
    logreg.fit(vec_train, sublabel_train)

    # Predictions
    sublabel_pred_test = logreg.predict(vec_test)
    sublabel_pred_train = logreg.predict(vec_train)

    classes = list(set(sublabel_test))

    cnf_matrix = metrics.confusion_matrix(sublabel_test, sublabel_pred_test, labels=classes)
    cnf_matrix = pd.DataFrame(cnf_matrix, columns=classes)
    cnf_matrix.rename(index={i: k for i, k in enumerate(classes)}, inplace=True)

    print("\n------------------ ", lb, " --------------------\n")
    print("Note: Columns (predictions) vs. Rows (actuals)\n")
    print(cnf_matrix, '\n')

    accuracy = metrics.accuracy_score(sublabel_test, sublabel_pred_test)
    precision_score = metrics.precision_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)
    recall_score = metrics.recall_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)
    f1_score = metrics.f1_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)

    print("\n------------------ Test --------------------\n")
    print("Accuracy:", accuracy)
    print("Precision:", precision_score)
    print("Recall:", recall_score)
    print("Unweighted Average F1 Score:", f1_score)

    accuracy = metrics.accuracy_score(sublabel_train, sublabel_pred_train)
    precision_score = metrics.precision_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)
    recall_score = metrics.recall_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)
    f1_score = metrics.f1_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)

    print("\n------------------ Train --------------------\n")
    print("Accuracy:", accuracy)
    print("Precision:", precision_score)
    print("Recall:", recall_score)
    print("Unweighted Average F1 Score:", f1_score)


# 2-stage model: first Pr(main labels) -> add to the vector -> 306 dimension -> new logistic model

# ### 3.4 Decision Tree: Main Labels

# In[97]:


vec_train, vec_test, label_train, label_test = train_test_split(
    df_vec, df['label'], test_size=0.25, random_state=0, stratify=df['label'])

# Create Decision Tree classifer object
clf = DecisionTreeClassifier(criterion="entropy", class_weight='balanced')

# Train Decision Tree Classifer
clf = clf.fit(vec_train, label_train)

# Predict the response for test dataset
label_pred_test = clf.predict(vec_test)
label_pred_train = clf.predict(vec_train)

accuracy = metrics.accuracy_score(label_test, label_pred_test)
print("\n------------------ Test --------------------\n")
print("Accuracy:", accuracy)

accuracy = metrics.accuracy_score(label_train, label_pred_train)
print("\n------------------ Train --------------------\n")
print("Accuracy:", accuracy)


# Explainable but Random forest is better for modelling
# Find out key dimensions

# ### 3.5 Performance Comparison

# ### 3.5.1 **1000 data samples**
#
# || LGR-Main Lables | LGR-Sublabels| LGR-Conditional | Decision Tree-Labels |
# |-----|--------|--------|--------|--------|
# |Accuracy| 0.684 | 0.544 | 0.443 - 0.854 | 0.384 |
#
#
#  **Observation:** Poor calssification performance of ENTY sublabels given the main label, i.e., ENTY.
#
# **Conditioanl Classification**
#
#
# |   | ABBR | HUM | DESC | NUM | LOC | ENTY|
# |---|---|---|---|---|---|---|
# |Accuracy| 0.800 | 0.854 | 0.755 | 0.737 | 0.692 | 0.443 |

# ### 3.5.2 **5452 data samples**
#
# || LGR-Main Lables | LGR-Sublabels| LGR-Conditional | Decision Tree-Labels |
# |-----|--------|--------|--------|--------|
# |Accuracy| 0.743 | 0.650 | 0.689 - 0.909 | 0.428 |
#
#
#  **Observation:** Poor calssification performance of ENTY sublabels given the main label, i.e., ENTY.
#
#  **Conditioanl Classification**
#
#
# |   | ABBR | HUM | DESC | NUM | LOC | ENTY|
# |---|---|---|---|---|---|---|
# |Accuracy| 0.909 | 0.882 | 0.818 | 0.808 | 0.899 | 0.687 |

# ### 3.5.3 **5452 data samples, Balanced training and Stratified sets**
#
# || LGR-Main Lables | LGR-Sublabels| LGR-Conditional | Decision Tree-Labels |
# |-----|--------|--------|--------|--------|
# |Accuracy| 0.733 | 0.643 | 0.716 - 0.954 | 0.440 |
#
#
#  **Observation:** Poor calssification performance of ENTY sublabels given the main label, i.e., ENTY.
#
#  **Conditioanl Classification**
#
#
# |   | ABBR | HUM | DESC | NUM | LOC | ENTY|
# |---|---|---|---|---|---|---|
# |Accuracy| 0.954 | 0.925 | 0.859 | 0.844 | 0.919 | 0.716 |

# - vertical form: how long it takes to train.
# - F1
# - imbalanced dataset
# - balanced = true , or auto
# - Stratified train test split
# - oversampling: repeating the infrequent test data

# Next Step- USE (just encoder)
# BERT (enc/dec) - generate sentence

# ## 4. USE

# ### 4.1 Vector Generation

# In[135]:

module_url = "https://tfhub.dev/google/universal-sentence-encoder-large/3"
# Import the Universal Sentence Encoder's TF Hub module
embed = hub.Module(module_url)

# Compute a representation for each message, showing various lengths supported.
messages = list(df['question'])

with tf.Session() as session:
    session.run([tf.global_variables_initializer(), tf.tables_initializer()])
    message_embeddings = session.run(embed(messages))


# In[50]:


message_embeddings.shape


# ### 4.2 Logistic Regression: Main Lable Classification

# In[98]:


# import sklearn

vec_train, vec_test, label_train, label_test = train_test_split(
    message_embeddings, df['label'], test_size=0.25, random_state=0, stratify=df['label'])

# import the class

# instantiate the model (using the default parameters)
logreg = LogisticRegression(solver='liblinear', multi_class='auto', class_weight='balanced')

# fit the model with data
logreg.fit(vec_train, label_train)

# Predictions
label_pred_test = logreg.predict(vec_test)
label_pred_train = logreg.predict(vec_train)


# In[99]:


classes = list(set(label_test))

# import the metrics class

cnf_matrix = metrics.confusion_matrix(label_test, label_pred_test, labels=classes)
cnf_matrix = pd.DataFrame(cnf_matrix, columns=classes)
cnf_matrix.rename(index={i: k for i, k in enumerate(classes)}, inplace=True)

print("Note: Columns (predictions) vs. Rows (actuals)\n")
print(cnf_matrix)


# In[102]:


accuracy = metrics.accuracy_score(label_test, label_pred_test)
precision_score = metrics.precision_score(label_test, label_pred_test, average='macro', labels=classes)
recall_score = metrics.recall_score(label_test, label_pred_test, average='macro', labels=classes)
f1_score = metrics.f1_score(label_test, label_pred_test, average='macro', labels=classes)

print("Accuracy:", accuracy)
print("Precision:", precision_score)
print("Recall:", recall_score)
print("Unweighted Average F1 Score:", f1_score)


# In[103]:


accuracy = metrics.accuracy_score(label_train, label_pred_train)
precision_score = metrics.precision_score(label_train, label_pred_train, average='macro', labels=classes)
recall_score = metrics.recall_score(label_train, label_pred_train, average='macro', labels=classes)
f1_score = metrics.f1_score(label_train, label_pred_train, average='macro', labels=classes)

print("Accuracy:", accuracy)
print("Precision:", precision_score)
print("Recall:", recall_score)
print("Unweighted Average F1 Score:", f1_score)


# ### 4.3 Logistic Regression: Sublabel Classification

# In[104]:


vec_train, vec_test, sublabel_train, sublabel_test = train_test_split(
    message_embeddings, df['sublabel'], test_size=0.25, random_state=0, stratify=df['sublabel'])

# instantiate the model (using the default parameters)
logreg = LogisticRegression(solver='liblinear', multi_class='auto', class_weight='balanced')

# fit the model with data
logreg.fit(vec_train, sublabel_train)

# Predictions
sublabel_pred_test = logreg.predict(vec_test)
sublabel_pred_train = logreg.predict(vec_train)

classes = list(set(sublabel_test))

cnf_matrix = metrics.confusion_matrix(sublabel_test, sublabel_pred_test, labels=classes)
cnf_matrix = pd.DataFrame(cnf_matrix, columns=classes)
cnf_matrix.rename(index={i: k for i, k in enumerate(classes)}, inplace=True)

print("Note: Columns (predictions) vs. Rows (actuals)\n")
print(cnf_matrix, "\n")

accuracy = metrics.accuracy_score(sublabel_test, sublabel_pred_test)
precision_score = metrics.precision_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)
recall_score = metrics.recall_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)
f1_score = metrics.f1_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)

print("Accuracy:", accuracy)
print("Precision:", precision_score)
print("Recall:", recall_score)
print("Unweighted Average F1 Score:", f1_score)


# In[105]:


accuracy = metrics.accuracy_score(sublabel_train, sublabel_pred_train)
precision_score = metrics.precision_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)
recall_score = metrics.recall_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)
f1_score = metrics.f1_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)

print("Accuracy:", accuracy)
print("Precision:", precision_score)
print("Recall:", recall_score)
print("Unweighted Average F1 Score:", f1_score)


# ### 4.4 Logistic Regression: Conditional Classification

# In[106]:


for lb in ['ABBR', 'HUM', 'DESC', 'NUM', 'LOC', 'ENTY']:

    vec_train, vec_test, sublabel_train, sublabel_test = train_test_split(
        message_embeddings[df['label'] == lb], df[df['label'] == lb]['sublabel'],
        test_size=0.25, random_state=0, stratify=df[df['label'] == lb]['sublabel'])

    # instantiate the model (using the default parameters)
    logreg = LogisticRegression(solver='liblinear', multi_class='auto', class_weight='balanced')

    # fit the model with data
    logreg.fit(vec_train, sublabel_train)

    # Predictions
    sublabel_pred_test = logreg.predict(vec_test)
    sublabel_pred_train = logreg.predict(vec_train)

    classes = list(set(sublabel_test))

    cnf_matrix = metrics.confusion_matrix(sublabel_test, sublabel_pred_test, labels=classes)
    cnf_matrix = pd.DataFrame(cnf_matrix, columns=classes)
    cnf_matrix.rename(index={i: k for i, k in enumerate(classes)}, inplace=True)

    print("\n------------------ ", lb, " --------------------\n")
    print("Note: Columns (predictions) vs. Rows (actuals)\n")
    print(cnf_matrix, '\n')

    accuracy = metrics.accuracy_score(sublabel_test, sublabel_pred_test)
    precision_score = metrics.precision_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)
    recall_score = metrics.recall_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)
    f1_score = metrics.f1_score(sublabel_test, sublabel_pred_test, average='weighted', labels=classes)

    print("\n------------------ Test --------------------\n")
    print("Accuracy:", accuracy)
    print("Precision:", precision_score)
    print("Recall:", recall_score)
    print("Unweighted Average F1 Score:", f1_score)

    accuracy = metrics.accuracy_score(sublabel_train, sublabel_pred_train)
    precision_score = metrics.precision_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)
    recall_score = metrics.recall_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)
    f1_score = metrics.f1_score(sublabel_train, sublabel_pred_train, average='weighted', labels=classes)

    print("\n------------------ Train --------------------\n")
    print("Accuracy:", accuracy)
    print("Precision:", precision_score)
    print("Recall:", recall_score)
    print("Unweighted Average F1 Score:", f1_score)


# ### 4.5 Decision Tree: Main Labels

# In[110]:

vec_train, vec_test, label_train, label_test = train_test_split(
    message_embeddings, df['label'], test_size=0.25, random_state=0, stratify=df['label'])

# Create Decision Tree classifer object
clf = DecisionTreeClassifier(criterion="entropy", class_weight='balanced')

# Train Decision Tree Classifer
clf = clf.fit(vec_train, label_train)

# Predict the response for test dataset
label_pred_test = clf.predict(vec_test)
label_pred_train = clf.predict(vec_train)

accuracy = metrics.accuracy_score(label_test, label_pred_test)
print("\n------------------ Test --------------------\n")
print("Accuracy:", accuracy)

accuracy = metrics.accuracy_score(label_train, label_pred_train)
print("\n------------------ Train --------------------\n")
print("Accuracy:", accuracy)


# ### 4.6 Performance

# #### 4.6.1 **Accuracy - 5452 data samples**
#
# || LGR-Main Lables | LGR-Sublabels| LGR-Conditional | Decision Tree-Labels |
# |-----|--------|--------|--------|--------|
# |Spacy| 0.743 | 0.650 | 0.689 - 0.909 | 0.428 |
# |USE| 0.896 | 0.728 | 0.712 - 0.895 | 0.717 |
#
#
#  **Conditioanl Classification**
#
#
# |   | ABBR | HUM | DESC | NUM | LOC | ENTY|
# |---|---|---|---|---|---|---|
# |Spacy| 0.909 | 0.882 | 0.818 | 0.808 | 0.899 | 0.687 |
# |USE| 0.727 | 0.895 | 0.835 | 0.790 | 0.885 | 0.712 |

# #### 4.6.2 **Accuracy - 5452 data samples, Balanced/Stratified**
#
# || LGR-Main Lables | LGR-Sublabels| LGR-Conditional | Decision Tree-Labels |
# |-----|--------|--------|--------|--------|
# |Spacy| 0.733 | 0.643 | 0.716 - 0.954 | 0.440 |
# |USE| 0.902 | 0.739 | 0.770 - 0.957 | 0.718 |
#
#
#  **Conditioanl Classification**
#
#
# |   | ABBR | HUM | DESC | NUM | LOC | ENTY|
# |---|---|---|---|---|---|---|
# |Spacy|  0.954 | 0.925 | 0.859 | 0.844 | 0.919 | 0.716 |
# |USE| 0.818 | 0.957 | 0.859 | 0.879 | 0.923 | 0.770 |

# ## 5. Keras

# In[136]:


def UniversalEmbedding(x):
    return embed(tf.squeeze(tf.cast(x, tf.string)), signature="default", as_dict=True)["default"]


input_text = layers.Input(shape=(1,), dtype=tf.string)
embedding = layers.Lambda(UniversalEmbedding, output_shape=(512,))(input_text)
dense = layers.Dense(256, activation='relu')(embedding)
pred = layers.Dense(6, activation='softmax')(dense)
model = Model(inputs=[input_text], outputs=pred)
model.compile(loss='categorical_crossentropy', optimizer='adam', metrics=['accuracy'])

model.summary()

text_train, text_test, label_train, label_test = train_test_split(
    df['question'], df['label'], test_size=0.25, random_state=0, stratify=df['label'])

label_train = np.asarray(pd.get_dummies(label_train), dtype=np.int8)
label_test = np.asarray(pd.get_dummies(label_test), dtype=np.int8)

with tf.Session() as session:
    K.set_session(session)
    session.run(tf.global_variables_initializer())
    session.run(tf.tables_initializer())
    history = model.fit(text_train,
                        label_train,
                        validation_data=(text_test, label_test),
                        epochs=10,
                        batch_size=32)
    model.save_weights('./model.h5')


# ### 5.1 Performance

# **Accuracy - 5452 data samples, Balanced/Stratified**
#
# || LGR-Main Lables | LGR-Sublabels| LGR-Conditional | Decision Tree-Labels | NN |
# |-----|--------|--------|--------|--------|-------|
# |Spacy| 0.733 | 0.643 | 0.716 - 0.954 | 0.440 | N.A. |
# |USE| 0.902 | 0.739 | 0.770 - 0.957 | 0.718 | 0.918|

# ## 6. Data Story


# In[178]:

labels = ['HUM', 'DESC', 'ENTY', 'LOC', 'ABBR', 'NUM']

word_count = dict.fromkeys(labels)

for label in labels:

    word_count_labeled = Counter()

    for idx, row in df[df['label'] == label].iterrows():
        doc = nlp(row['question'])
        word_count_labeled += Counter([t.text for t in doc
                                       if t.pos_ in
                                       ['NOUN', 'PROPN', 'ADJ', 'ADV', 'PRON']])

    word_count[label] = dict(word_count_labeled)

print(word_count)


# In[182]:


df_word_count = pd.DataFrame(word_count)

df_word_top_five = pd.DataFrame()

for col in df_word_count.columns:
    df_word_top_five = pd.concat([df_word_top_five, df_word_count[col].
                                  sort_values(ascending=False).head(5) /
                                  df_word_count[col].sum() * 100], axis=1, sort=False)


plt.rcParams['figure.figsize'] = [20, 5]
plt.rcParams.update({'font.size': 14})
df_word_top_five.plot.bar(width=0.8)
plt.xlabel('Top 5 Words in each Category')
plt.ylabel('% of Counts in each Category')
plt.xticks()
plt.show()


# In[150]:

df_tidy = pd.melt(df_word_top_five.reset_index(), id_vars='index')
sns.barplot(x='index', y='value', hue='variable', data=df_tidy)
plt.show()


df_label_dummy = pd.get_dummies(df.label)
df_vec_label = pd.concat([df_vec, df_label_dummy], axis=1)
df_vec_label_corr = df_vec_label.corr(method='pearson')
# df_vec_label_corr.style.background_gradient(cmap='coolwarm', axis=None)


fig, ax = plt.subplots(figsize=(3, 20))

ax = sns.heatmap(
    df_vec_label_corr,
    vmin=-1, vmax=1, center=0,
    cmap=sns.diverging_palette(20, 220, n=10),
    square=False, ax=ax
)
ax.set_xticklabels(
    ax.get_xticklabels(),
    rotation=90,
    horizontalalignment='right'
)


# In[ ]:



# test_doctests
import pytest  # noqa
import doctest

import qary.clibot

import qary.skills.basebots
import qary.skills.eliza_bots
import qary.skills.glossary_bots
import qary.skills.qa_bots
import qary.skills.faq_bots

import qary.etl.glossaries
import qary.etl.scrape_wikipedia
import qary.etl.yml
import qary.etl.elastic
import qary.etl.knowledge_extraction
import qary.etl.qa_datasets

import qary.scores.semantics_score
# import qary.template_generators


__author__ = "SEE AUTHORS.md"
__copyright__ = "Hobson Lane"
__license__ = "The Hippocratic License, see LICENSE.txt (MIT + Do no Harm)"

DOCTEST_KWARGS = dict(
    optionflags=doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE,
    verbose=True)


def test_basebots():
    results = doctest.testmod(qary.skills.basebots, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 0


def test_eliza_bots():
    results = doctest.testmod(qary.skills.eliza_bots, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 0


def test_clibot():
    results = doctest.testmod(qary.clibot, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 0


def test_glossaries():
    results = doctest.testmod(qary.etl.glossaries, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 4


def test_qa_datasets():
    results = doctest.testmod(qary.etl.qa_datasets, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 0


def test_qa_bots():
    results = doctest.testmod(qary.skills.qa_bots, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 0


def test_scrape_wikipedia():
    results = doctest.testmod(qary.etl.scrape_wikipedia, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 2


def test_faq_bots():
    results = doctest.testmod(qary.skills.faq_bots, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 0


# def test_yml():
#     results = doctest.testmod(qary.etl.yml, optionflags=doctest.ELLIPSIS |
#                               doctest.NORMALIZE_WHITESPACE, verbose=True)
#     assert results.failed < 1
#     assert results.attempted > 2


def test_semantics_score():
    results = doctest.testmod(qary.scores.semantics_score, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted >= 1


# def test_template_generators():
#     results = doctest.testmod(qary.template_generators, **DOCTEST_KWARGS)
#     assert results.failed < 1
#     assert results.attempted >= 1


def test_glossary_bots():
    results = doctest.testmod(qary.skills.glossary_bots, **DOCTEST_KWARGS)
    assert results.failed < 1
    assert results.attempted > 2


def test_etl_elastic():
    results = doctest.testmod(qary.etl.elastic, **DOCTEST_KWARGS)
    assert results.failed == 0
    assert results.attempted >= 0


def test_etl_knowledge_extraction():
    results = doctest.testmod(qary.etl.knowledge_extraction, **DOCTEST_KWARGS)
    assert results.failed == 0
    assert results.attempted >= 0



from django.db import models
from django.urls import reverse

# Create your models here.


class Purpose(models.Model):
    purpose = models.CharField(max_length=256)

    def __str__(self):
        return self.purpose

    def get_absolute_url(self):
        return reverse('swot:detail', kwargs={'pk': self.pk})


class SWOT(models.Model):
    strength = models.CharField(max_length=256, blank=True, default=None)
    strength_rate = models.PositiveIntegerField(null=True, default=0)
    weaknesses = models.CharField(max_length=256, blank=True, default=None)
    weaknesses_rate = models.PositiveIntegerField(null=True, default=0)
    oportinuties = models.CharField(max_length=256, blank=True, default=None)
    oportinuties_rate = models.PositiveIntegerField(null=True, default=0)
    threats = models.CharField(max_length=256, blank=True, default=None)
    threats_rate = models.PositiveIntegerField(null=True, default=0)
    strategy = models.TextField(blank=True, default=None)
    purpose = models.ForeignKey(
        Purpose, on_delete=models.CASCADE, related_name='s')

    def __str__(self):
        return self.strength

    # def get_absolute_url(self):
    #     return reverse('swot:detail', kwargs={'pk': self.pk})



import re


RE_DICT_VALUE_HASHTAGS = r'((\s*[^:]*[:])\s+([^":#]*)(\s*#[-+/ _\w\d]+))'
RE_EOL_HASHTAG = r'\s+[#][-+/_\w\d]+\s*$'


def translate_yml_lines(path):
    """ Yield lines of a yaml file, with changes made to accomidate extensions, like unquoted hashtags.

    Transformations:
       any string: any string #hashtag #ht #etc
         => any string: "any string #hashtag #ht #etc"

    >>> import io
    >>> lines = 'hello big: World, now. #good #bye \nnot: a hash\n# comment\n\n'
    >>> list(translate_yaml_lines(io.StringIO(lines))
    ['hello big: "World, now. #good #bye"']
    """
    with open(path, 'rt') as fin:
        for line in fin:
            match = re.match(r'\s+[#][-+/_\w\d]+\s*$', line)
            groups = match.groups() if match else [None]
            if groups[0] == line and len(groups) == 4:
                translated = groups[1] + ' "' + groups[2] + groups[3] + '"'
                print(translated)
                yield
            else:
                yield line


def find_hashtags(s, pattern=r'\s*#[\w\d_-]+'):
    """ Find twitter-style tags embedded within a string.

    >>> d = find_hashtags("Find #this hashtag #too #sarcasm-not.")
    >>> d['cleaned']
    'Find hashtag.'
    >>> d['hashtags']
    ['#sarcasm-not', '#this', '#too']
    """
    s = s or ''
    hashtags = re.findall(pattern, s) or []
    hashtags = sorted(set([t.strip() for t in hashtags]))
    cleaned = re.sub(pattern, '', s)
    return {'cleaned': cleaned, 'hashtags': hashtags}



import logging

from ..spacy_language_model import nlp  # noqa

log = logging.getLogger(__name__)


def iou(a, b):
    """ Crude character vector overlap measure of string similarity

    >>> iou('Hello', 'World')
    0.285...
    """
    a, b = set(a.lower().strip()), set(b.lower().strip())
    return len(a & b) / len(a | b)


def semantics(reply, stmt=None, **kwargs):
    """ Compute word2vec docvec cosine similarity (fall back to character IOU)

    >>> semantics('Hello world!', 'Goodbye big earth!') > .5
    True
    """
    global nlp
    nlp = kwargs.get('nlp', nlp)
    if kwargs is None or nlp is None or not stmt or not reply:
        return 0.0

    reply_doc, stmt_doc = nlp(str(reply)), nlp(str(stmt))

    if not reply_doc or not stmt_doc or not reply_doc.has_vector or not stmt_doc.has_vector:
        # FIXME: levenshtien would be better or fuzzywuzzy
        return iou(reply, stmt)

    cos_sim = reply_doc.similarity(stmt_doc)
    log.debug(f'cos_sim={cos_sim}')
    return cos_sim



from django.db import models
from django.urls import reverse


class Post(models.Model):
    author = models.CharField(max_length=200)
    title = models.CharField(max_length=200)
    text = models.TextField()

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('post_detail', kwargs={pk: self.pk})




#!/usr/bin/env python
from ncclient import manager 
host = "5.1.105.105" 
user = "moh@hrinsvpn"
pw = "123321"
port = 2022
with manager.connect_ssh(host=host, port=port,
username=user, look_for_keys=False,
password=pw,
allow_agent=False,
hostkey_verify=False) as m:
c = m.get_config(source='running').data_xml with open("%s.xml" % host, 'w') as f:
f.write(c)



""" Pattern and template based chatbot dialog engines """
import logging

# import pandas as pd

from qary.etl import glossaries
from qary import spacy_language_model
from qary.etl import knowledge_extraction as extract
from qary.skills.basebots import ContextBot

log = logging.getLogger(__name__)
nlp = spacy_language_model.load('en_core_web_md')


def capitalizations(s):
    return (s, s.lower(), s.upper(), s.title())


class Bot(ContextBot):
    """ Bot that can reply with definitions from glossary yml files in data/faq/glossary-*.yml

    >>> bot = Bot()
    >>> bot.reply('allele')
    [(0.05, "I don't understand...")]
    >>> bot.reply('What is an Allele?')
    [(1.0, 'A variant form of a given gene...')]
    >>> bot.reply('What is a nucleotide?')
    [(0.94, 'The basic building blocks of DNA and RNA...')]
    """

    def __init__(self, domains=('dsdh',)):
        """ Load glossary from yaml file indicated by list of domain names """
        global nlp
        super().__init__(self)
        self.nlp = nlp
        self.glossary = glossaries.load(domains=domains)['cleaned']
        self.vector = dict()
        self.vector['term'] = glossaries.term_vector_dict(self.glossary.keys())
        self.vector['definition'] = glossaries.term_vector_dict(self.glossary.values(), self.glossary.keys())

        self.synonyms = {term: term for term in self.glossary}
        # create reverse index of synonyms to canonical terms
        # for term, d in self.glossary.items():
        #     self.synonyms.update(dict(zip(capitalizations(term), [term] * 4)))
        #     acro = d['acronym']
        #     if acro:
        #         self.synonyms.update(dict(zip(capitalizations(acro), [term] * 4)))

    def reply(self, statement):
        """ Suggest responses to a user statement string with [(score, reply_string)..]"""
        responses = []
        extracted_term = extract.whatis(statement) or extract.whatmeans(statement) or ''
        if extracted_term:
            for i, term in enumerate(capitalizations(extracted_term)):
                normalized_term = self.synonyms.get(term, term)
                if normalized_term in self.glossary:
                    responses.append((1 - .02 * i, self.glossary[normalized_term]['definition']))
        else:
            responses = [(0.05, "I don't understand. That doesn't sound like a question I can answer using my glossary.")]
        if not len(responses):
            responses.append((0.25, f"My glossaries and dictionaries don't seem to contain that term ('{extracted_term}')."))
        return responses
